`define SIMULATION 1

`timescale 10ns/100ps

// MODEL_TECH
module  mem_tb();
  localparam CLK_CYCLES_RESET = 50;
  localparam CLK_CYCLES_SIM   = 20000;
  localparam DEFAULT_WORD     = 32'hDEADBEEF;
  localparam SINGLE_WORD      = 32'hBADC0FFE;
  localparam WORD_SIZE_MEM    = 16;
  localparam AHB_SIZE_ADDR    = 32;
  localparam AHB_SIZE_DATA    = 32;

  reg clk;
  reg reset_n;
  integer i;

  reg hsel;
  reg [AHB_SIZE_ADDR-1:0] haddr;
  reg [AHB_SIZE_DATA-1:0] hwdata;
  reg hwrite;
  reg [2:0] hsize;
  reg [1:0] htrans;
  reg hready;
  wire [AHB_SIZE_DATA-1:0] hrdata;
  wire hreadyout;
  wire hresp;

  assign hready = hreadyout;

  initial begin
    clk <= 1'b0;
    forever #10 clk <= ~clk;
  end

  task reset_routine;
    begin
      $display("Starting reset...");
      reset_n <= 1;
      for(i=0; i<CLK_CYCLES_RESET; i++)
        @(posedge clk);
      reset_n <= 0;
      for(i=0; i<CLK_CYCLES_RESET; i++)
        @(posedge clk);
      reset_n <= 1;
      $display("Finishing reset...\n");
    end
  endtask

  ahb3lite_sram1rw #(
    .MEM_SIZE(0),
    .MEM_DEPTH(WORD_SIZE_MEM),
    .HADDR_SIZE(8),
    .HDATA_SIZE(32),
    .TECHNOLOGY("GENERIC"),
    .REGISTERED_OUTPUT("NO") 
  )
  ahb_memory
  (
    .HRESETn(reset_n),
    .HCLK(clk),
    .HSEL(hsel),
    .HADDR(haddr),
    .HWDATA(hwdata),
    .HRDATA(hrdata),
    .HWRITE(hwrite),
    .HSIZE(hsize),
    .HBURST(1'b0),
    .HPROT(4'd3),
    .HTRANS(htrans),
    .HREADYOUT(hreadyout),
    .HREADY(hready),
    .HRESP(hresp)
  );

  initial begin
    reset_routine();

    hsel    <= 1'b0;
    haddr   <= 32'd0;
    hwdata  <= 32'd0;
    hwrite  <= 1'b0;
    hsize   <= 3'd2;  // Word
    htrans  <= 2'b00; // IDLE
    @(posedge clk);

    // Initialize the memory
    for(i=0; i<WORD_SIZE_MEM; i++) begin  
      hsel    <= 1'b1;
      haddr   <= i*4;
      hwrite  <= 1'b1;
      hsize   <= 3'd2; // Word
      htrans  <= 2'd2; // NON_SEQ
      hwdata  <= DEFAULT_WORD;
      @(posedge clk); // Address phase
      @(posedge clk); // Data phase
    end

    hsel    <= 1'b0;
    haddr   <= 32'd0;
    hwdata  <= 32'd0;
    hwrite  <= 1'b0;
    hsize   <= 3'd2;  // Word
    htrans  <= 2'b00; // IDLE
    @(posedge clk);
    @(posedge clk);
    @(posedge clk);
      
    hsel    <= 1'b1;
    haddr   <= 32'd2;
    hwrite  <= 1'b1;
    hsize   <= 3'd0; // Byte
    htrans  <= 2'd2; // IDLE  
    @(posedge clk);
    hwrite  <= 1'b0;
    hwdata  <= SINGLE_WORD;
    @(posedge clk);
    hsel    <= 1'b0;
    @(posedge clk);

    for(i=0; i<CLK_CYCLES_SIM/2; i++) begin
      @(posedge clk);
    end
    
    $display("\nFinished at time %t after %d iterations",$time,i);
    $finish;
  end

endmodule
